import pylab
from matplotlib import pyplot
from PlotInfo import *

class Bar(PlotInfo):
    """
    A bar chart consisting of a single series of bars.
    """

    def __init__(self):
        PlotInfo.__init__(self, "bar")
        self.width=0.8
        self.color="black"
        self.edgeColor=None
        self.hatch=None
        self.align="center"
    
    def draw(self, axis, transform=None):
        PlotInfo.draw(self, axis)

        return self._draw(axis, transform)

    def _draw(self, axis, transform=None):
        kwdict = self.getAttributes()

        if transform:
            xValues = [transform.transform((x,0))[0] for x in self.xValues]
        else:
            xValues = self.xValues

        return [[axis.bar(xValues, self.yValues, **kwdict)[0]],
                [self.label]]

    def getAttributes(self):
        kwdict = {}
        kwdict["color"] = self.color
        kwdict["label"] = self.label
        kwdict["width"] = self.width
        kwdict["align"] = self.align

        if self.hatch is not None:
            kwdict["hatch"] = self.hatch
            print >>sys.stderr, "WARNING: Setting hash for bar charts only seems to work when exporting to svg, png, or pdf"

        if self.edgeColor is not None:
            kwdict["edgecolor"] = self.edgeColor

        return kwdict



__all__ = ["Bar", "Plot", "PlotLayout", "WeightedPlotLayout",
           "Scatter", "ClusteredBars", "PlotInfo", "Line", "Utils", 
           "StackedBars", "Label", "StackedLines"]

for __mySrcDir__ in __all__:
    exec("from %s import *" % (__mySrcDir__))



import pylab
from matplotlib import pyplot
from matplotlib.patches import Rectangle
from PlotInfo import *
from Line import *
import sys

class StackedLines(PlotInfo):
    """
    Lines stacked one atop the other, with colors filling in the gaps
    """

    def __init__(self):
        PlotInfo.__init__(self, "stacked lines")

        self.lines = []
        self.colors = []
    
    def addLine(self, line, color="white"):
        self.lines.append(line)
        self.colors.append(color)

    def draw(self, axis, transform=None):
        PlotInfo.draw(self, axis)
        
        plotHandles = []
        plotLabels = []
        
        xValues = None
        yValues = []
        
        for line in self.lines:
            if xValues == None:
                xValues = line.xValues
            yValues.append(line.yValues)
        
        yDataStacked = self._cumulativeSum(yValues)
        
        axis.fill_between(xValues, 0, yDataStacked[0], 
                          facecolor=self.colors[0],
                          linestyle=self.lines[0].lineStyle)
        # Since fill_between doesn't have legend support, will have to create a
        # proxy artist for it. See
        # http://matplotlib.sourceforge.net/users/legend_guide.html#using-proxy-artist
        proxyArtist = Rectangle((0,0), 1, 1, color=self.colors[0])
        plotHandles.append(proxyArtist)
        plotLabels.append(self.lines[0].label)
        
        for i in xrange(len(yDataStacked) - 1):
            axis.fill_between(xValues, yDataStacked[i], 
                              yDataStacked[i + 1], 
                              facecolor=self.colors[i + 1],
                              linestyle=self.lines[i+1].lineStyle)
            proxyArtist = Rectangle((0,0), 1, 1, color=self.colors[i+1])
            plotHandles.append(proxyArtist)
            plotLabels.append(self.lines[i+1].label)
        return [plotHandles, plotLabels]
    
    def _cumulativeSum(self, yValuesLists):
        output = []

        for i in xrange(len(yValuesLists)):
            currentList = yValuesLists[i]

            if i == 0:
                output.append(currentList)
                continue
            else:
                prevList = output[i-1]
                newList = [currentList[i] + prevList[i] for i in xrange(len(currentList))]
                output.append(newList)
        return output    



import os
import sys
import copy

class PlotInfo:
    def __init__(self, plotType):
        self.plotType = plotType

        self.xValues = []
        self.yValues = []

        self.xTickLabels = None
        self.yTickLabels = None
        self.xTickLabelPoints = None
        self.yTickLabelPoints = None
        self.xTickLabelProperties = {}
        self.yTickLabelProperties = {}
        
        self.label = None
        
        self.yMins = []
        self.yMaxes = []
        self.yErrors = []
        
        self.autosort = True

        self.xLimits = None
        
    def __str__(self):
        return str(self.__dict__)
    
    def setXTickLabelProperties(self, **propList):
        self._setTickLabelProperties(self.xTickLabelProperties, propList)
        
    def setYTickLabelProperties(self, **propList):
        self._setTickLabelProperties(self.yTickLabelProperties, propList)
    
    def _setTickLabelProperties(self, tickPropsDict, propList):
        # Going to restrict the set of properties that can be modified so they
        # don't mess with the rest of the system
        
        validProperties = ["alpha", "backgroundColor", "color", \
            "horizontalalignment", "linespacing", "multialignment", \
            "rotation", "stretch", "style", 
            "verticalalignment", "weight"]
        
        for (key, val) in propList.items():
            if key not in validProperties:
                print >>sys.stderr, "Tick label property '%s' is not currently supported" % (key)
            else:
                tickPropsDict[key] = val
        
    def split(self, pieces):
        elements = []
        
        numXVals = len(self.xValues)
        
        valChunkSize = numXVals / pieces
        valChunkRemainder = numXVals % pieces
        
        for i in xrange(pieces):
            element = copy.deepcopy(self)

            if i < pieces - 1 or valChunkRemainder == 0:
                element.xValues = self.xValues[i * valChunkSize:(i+1) * valChunkSize]
                element.yValues = self.yValues[i * valChunkSize:(i+1) * valChunkSize]
            else:
                element.xValues = self.xValues[i * valChunkSize:]
                element.yValues = self.yValues[i * valChunkSize:]
            elements.append(element)
        return elements
        
    def draw(self, axis):
        if len(self.xValues) > 0 and self.autosort:
            # This is a total kludge --AR
            
            sortAsymmetricErrorBars = len(self.yMins) > 0 or \
                len(self.yMaxes) > 0
            sortSymmetricErrorBars = len(self.yErrors) > 0

            if sortAsymmetricErrorBars:
                if len(self.yMins) != len(self.yValues):
                    print >>sys.stderr, "You don't have an error bar for every point, some points will be truncated"

                zipped = zip(self.xValues, self.yValues, self.yMins, 
                             self.yMaxes)
                zipped.sort()
                self.xValues, self.yValues, self.yMins, self.yMaxes \
                    = zip(*zipped)
            elif sortSymmetricErrorBars:
                if len(self.yErrors) != len(self.yValues):
                    print >>sys.stderr, "You don't have an error bar for every point, some points will be truncated"

                zipped = zip(self.xValues, self.yValues, self.yErrors)
                zipped.sort()
                self.xValues, self.yValues, self.yErrors = zip(*zipped)
            else:
                zipped = zip(self.xValues, self.yValues)
                zipped.sort()
                self.xValues, self.yValues = zip(*zipped)
        
        if self.xTickLabels is not None:
            if self.xTickLabelPoints is None:
                axis.set_xticks(self.xValues[0:len(self.xTickLabels)])
            else:
                axis.set_xticks(self.xTickLabelPoints)
            
            axis.set_xticklabels(self.xTickLabels, **self.xTickLabelProperties)
        
        if self.yTickLabels is not None:
            if self.yTickLabelPoints is None:
                axis.set_yticks(self.yValues[0:len(self.yTickLabels)])
            else:
                axis.set_yticks(self.yTickLabelPoints)

            axis.set_yticklabels(self.yTickLabels, **self.yTickLabelProperties)

        self.drawErrorBars(axis)

    def drawErrorBars(self, axis, transform=None):
        errorBarKeywords = {}
        if hasattr(self, "ecolor"):
            errorBarKeywords["ecolor"] = self.ecolor
        elif hasattr(self, "color"):
            errorBarKeywords["ecolor"] = self.color

        if hasattr(self, "lineWidth"):
            errorBarKeywords["linewidth"] = self.lineWidth

        if hasattr(self, "lineStyle"):
            errorBarKeywords["linestyle"] = self.lineStyle

        if transform:
            errorBarKeywords['transform'] = transform + axis.transData

        errorBarKeywords["fmt"] = None
        
        if len(self.yMins) > 0 and len(self.yMaxes) > 0:
            numYVals = len(self.yValues)
            yMin = [self.yValues[i] - self.yMins[i] for i in xrange(numYVals)]
            yMax = [self.yMaxes[i] - self.yValues[i] for i in xrange(numYVals)]


            errorBarKeywords["yerr"] = [yMin, yMax]

            axis.errorbar(self.xValues, self.yValues, **errorBarKeywords) 
        elif len(self.yErrors) > 0:
            errorBarKeywords["yerr"] = self.yErrors
            axis.errorbar(self.xValues, self.yValues, **errorBarKeywords)



import pylab
from matplotlib import pyplot
from PlotInfo import *

class Scatter(PlotInfo):
    def __init__(self):
        PlotInfo.__init__(self, "scatter")
        self.marker = 'o'
        self.markerSize = 20
        self.color = "black"
            
    def draw(self, axis):
        kwdict = {}
        kwdict["marker"] = self.marker
        kwdict["label"] = self.label
        kwdict["color"] = self.color
        kwdict["s"] = self.markerSize
        
        return [[axis.scatter(self.xValues, self.yValues, **kwdict)], 
                [self.label]]
    



import pylab
from matplotlib import pylab
from PlotInfo import *
from Bar import *
import sys

class StackedBars(PlotInfo):
    """
    A stacked bar chart consisting of multiple series of bars with the 
    same X axis values
    """
    
    def __init__(self):
        PlotInfo.__init__(self, "stacked bar")
        
        self.bars = []
        self.spacing = 0
        self.width = 0.8
        
    def add(self, bar):
        if not isinstance(bar, Bar):
            print >>sys.stderr, "Can only add Bars to a StackedBars"
            sys.exit(1)
        
        if len(self.bars) == 0:
            # Fake having xValues
            self.xValues = bar.xValues
            self.yValues = bar.yValues
        bar.width = self.width
        self.bars.append(bar)

    def getXLabelLocations(self):
        if len(self.bars) == 0:
            return []
        else:
            numBarVals = len(self.bars[0].xValues)
            return range(numBarVals)

    def draw(self, axis, transform=None):
        self.xTickLabelPoints = self.getXLabelLocations()
        
        PlotInfo.draw(self, axis)

        return self._draw(axis, transform)

    def _draw(self, axis, transform=None):
        self.xTickLabelPoints = self.getXLabelLocations()

        plotHandles = []
        plotLabels = []

        if len(self.bars) == 0:
            return [plotHandles, plotLabels]

        numBars = len(self.bars)
        
        bottoms = [0 for i in xrange(len(self.xTickLabelPoints))]
        
        xVals = [i + i * self.spacing \
                     for i in xrange(len(self.bars[0].xValues))]

        if transform:
            xVals = [transform.transform((x,0))[0] for x in xVals]

        for bar in self.bars:
            attrs = bar.getAttributes()
            attrs['width'] = self.width

            currHandle = axis.bar(xVals, bar.yValues, bottom=bottoms, **attrs)
            
            bottoms = [bar.yValues[i] + bottoms[i] \
                           for i in xrange(len(self.xTickLabelPoints))]
            
            plotHandles.append(currHandle[0])
            plotLabels.append(bar.label)

        plotHandles.reverse()
        plotLabels.reverse()
        return [plotHandles, plotLabels]
